// Interactive Web-Based Pointillist Visualization of Hydrogenic Orbitals Using Jmol
//
// Shane P. Tully, Thomas M. Stitt, Robert D. Caldwell, Brian J. Hardock, 
// Robert M. Hanson, and Przemyslaw Maslak
//
// Department of Chemistry, the Pennsylvania State University, University
// Park, PA 16802, and Department of Chemistry, St. Olaf College, Northfield,
// MN 55057


// Thomas Stitt, Brian Hardock & Shane Tully
// Functions related to creating and graphing data for jmol and radial flot plots


// Constants
var DEFAULT_N = 1;
var DEFAULT_L = 0;
var DEFAULT_M = 0;
var DEFAULT_Z = 1;
var MAX_N_MAGNITUDE = 5;

var TOUR_LOCATION = "scripts/tour.spt";
var MODELS_LOCATION = "models/";

var INCREASE_EXTENT = 1;
var DECREASE_EXTENT = -1;

var DEFAULT_ZOOM_VALUE = 100;
var DEFAULT_TRANSPARENCY_VALUE = 99;
var DEFAULT_MC_CUTOFF = 0.0026;
var DEFAULT_MC_RESOLUTION = 5;

// The change in extent when changeExtent() is called
var DELTA_EXTENT = 5;
var DEFAULT_NUM_POINTS = 10000;
var DEFAULT_ADD_NUM_POINTS = 5000;

var NODEMENU_BACKGROUND = "rgba(134, 184, 36, 0.85)";
var NODEMENU_BACKGROUND_MOVING = "rgba(134, 184, 36, 0.5)";
var FLOT_TOOLTIP_BG = "#B9B3FF";

var DEFAULT_MC_NODE_COLOR = "yellow";
var DEFAULT_MC_NODE_TRANSPARENCY = 25;

// Keep track of what isosurface mode is displayed
var P_POINTILLIST_S = 0;
var P_POINTILLIST_M = 1;
var P_POINTILLIST_L = 2;
var D_POINTILLIST_S = 17;
var D_POINTILLIST_M = 18;
var D_POINTILLIST_L = 19;
var ISOSURFACE = 3;
var WIREFRAME = 4;
var CENTILE_50 = 14;
var CENTILE_90 = 15;
var CENTILE_95 = 16;
var CENTILE_50_M = 20;
var CENTILE_90_M = 21;
var CENTILE_95_M = 22;
var CROSS_SECTION_XY = 5;
var CROSS_SECTION_XZ = 6;
var CROSS_SECTION_YZ = 7;
var CONTOUR_XY = 8;
var CONTOUR_XZ = 9;
var CONTOUR_YZ = 10;
var RADIAL_SHELL = 12;
var DEFAULT_ISOSURFACE = P_POINTILLIST_M;

var SMALL_POINTILLIST_DOTS = 1;
var MEDIUM_POINTILLIST_DOTS = 2;
var LARGE_POINTILLIST_DOTS = 3;

// Globals
var seed;                                       // the seed for Jmol
var numPoints;                                  // the number of points for Jmol and flot

var flotPlaceHolder;                            // placeholder for flot on html page
var isosurfaceData;                             // isosurface data from Jmol's getIsosurfaceInfo() function
var isReady;                                    // For monte carlo data to be available Jmol must have plotted once.  This indicates if that is true.
var isAutomaticStepSize;                        // is autoStepSize is checked or not          
var curIsosurfaceMode;                          // The currently displayed isosurface mode
var lastRadius;                                 // The last used radius for isosurface shell mode
var isDisplayedFlotMsg;                         // boolean for message popup in flot

var is3D;                                       // Flag for if 3D contour or cross-section should be used
var calcTimer;                                  // Timer for showing the calculating menu
var mustMCUpdate;                               // Flag if MC MUST update
var isTourPlaying;

// For flot event handler issue
var previousRadius;                             // last non-marker click in the flot plot. Used to prevent multiple plotting (flashing) per plot
var previousMarker;                             // last marker click in the flot plot. Used to prevent multiple plotting (flashing) per plot

// For node menu
var isDragging = false;
var clickX, clickY, menuLeft, menuTop;
document.onmousedown = onMouseDown;
document.onmouseup = onMouseUp;


$(document).ready(function()
{
    // Init the sliders
    setupTransparencySlider();
    setupZoomSlider();
    setupSphericalNodeSlider();
    setupConicalNodeSlider();
    setupPlanarNodeSlider();
    
    // Init the isosurface type menu
    $("ul.sf-menu").superfish({ 
        delay: 500,
        animation: {opacity:'show',height:'show'},
        speed: 'fast'
    }); 
   
    // Init the orbital values in jmol_scripting
    updateOrbitalValues();
    
    // Init other stuff
    //extent = parseInt(document.getElementById("flotExtentText").value);
    isReady = false;
    previousMarker = -1;
    cutoff = 0;
    numPoints = DEFAULT_NUM_POINTS;
    flotPlaceHolder = "#flotGraph";
    updateSeed();
    isosurfaceData = null;
    curIsosurfaceMode = DEFAULT_ISOSURFACE;
    tourPaused = false;
    isDisplayedFlotMsg = false;
    mustMCUpdate = false;
    isTourPlaying = false;
    
    jmolSetPostCalcFunction("postJmolCalc()");
    
    isAutomaticRadius = document.getElementById("autoStepSize").checked;
    document.getElementById("stepSize").disabled = true;
        
    displayEmptyFlot(flotPlaceHolder);

    // Init a few jmol properties
    jmolSetToScale(false);
    jmolSetDotScale(2);
});


// Add event listeners
document.addEventListener("DOMContentLoaded", function()
{
        document.getElementById("showOrbital").addEventListener("click", actionListener, false);

        document.getElementById("decreaseFlotExtentButton").addEventListener("click", actionListener, false);
        document.getElementById("increaseFlotExtentButton").addEventListener("click", actionListener, false);
        
        document.getElementById("monteCarloRadioButton").addEventListener("change", actionListener, false);
        document.getElementById("theoreticalRadioButton").addEventListener("change", actionListener, false);
        
        document.getElementById("addPointsButton").addEventListener("click", actionListener, false);
        document.getElementById("waveFunctionCheckbox").addEventListener("click", actionListener, false);
        document.getElementById("electronDensityCheckbox").addEventListener("click", actionListener, false);
        document.getElementById("radialProbabilityDistributionCheckbox").addEventListener("click", actionListener, false);
        
        document.getElementById("decreaseN").addEventListener("click", actionListener, false);
        document.getElementById("increaseN").addEventListener("click", actionListener, false);  
        document.getElementById("decreaseL").addEventListener("click", actionListener, false);
        document.getElementById("increaseL").addEventListener("click", actionListener, false);
        document.getElementById("decreaseM").addEventListener("click", actionListener, false);
        document.getElementById("increaseM").addEventListener("click", actionListener, false);
        
        document.getElementById("pPointillistParent").addEventListener("click", actionListener, false);
        document.getElementById("pPointillistS").addEventListener("click", actionListener, false);
        document.getElementById("pPointillistM").addEventListener("click", actionListener, false);
        document.getElementById("pPointillistL").addEventListener("click", actionListener, false);
        document.getElementById("dPointillistParent").addEventListener("click", actionListener, false);
        document.getElementById("dPointillistS").addEventListener("click", actionListener, false);
        document.getElementById("dPointillistM").addEventListener("click", actionListener, false);
        document.getElementById("dPointillistL").addEventListener("click", actionListener, false);
        document.getElementById("isosurface").addEventListener("click", actionListener, false);
        document.getElementById("wireframe").addEventListener("click", actionListener, false);
        document.getElementById("centileParent").addEventListener("click", actionListener, false);
        document.getElementById("centile50").addEventListener("click", actionListener, false);
        document.getElementById("centile90").addEventListener("click", actionListener, false);
        document.getElementById("centile95").addEventListener("click", actionListener, false);
        document.getElementById("crossSectionParent").addEventListener("click", actionListener, false);
        document.getElementById("crossSectionXY").addEventListener("click", actionListener, false);
        document.getElementById("crossSectionXZ").addEventListener("click", actionListener, false);
        document.getElementById("crossSectionYZ").addEventListener("click", actionListener, false);
        document.getElementById("contourParent").addEventListener("click", actionListener, false);
        document.getElementById("contourXY").addEventListener("click", actionListener, false);
        document.getElementById("contourXZ").addEventListener("click", actionListener, false);
        document.getElementById("contourYZ").addEventListener("click", actionListener, false);

        document.getElementById("mark50").addEventListener("click", actionListener, false);
        document.getElementById("mark90").addEventListener("click", actionListener, false);
        document.getElementById("mark95").addEventListener("click", actionListener, false);
        
        document.getElementById("jmolWhiteBg").addEventListener("change", actionListener, false);
        document.getElementById("jmolBlackBg").addEventListener("change", actionListener, false);
        
        document.getElementById("jmolFitZoom").addEventListener("change", actionListener, false);
        document.getElementById("jmolFitScale").addEventListener("change", actionListener, false);
        
        document.getElementById("scaleX").addEventListener("click", actionListener, false);
        document.getElementById("scaleY").addEventListener("click", actionListener, false);
        document.getElementById("scaleZ").addEventListener("click", actionListener, false);

        document.getElementById("spin").addEventListener("click", actionListener, false);
        document.getElementById("axes").addEventListener("click", actionListener, false);
        document.getElementById("3d").addEventListener("change", actionListener, false);

        document.getElementById("manualStepSize").addEventListener("change", actionListener, false);
        document.getElementById("autoStepSize").addEventListener("change", actionListener, false);

        document.getElementById("reset").addEventListener("click", actionListener, false);
        document.getElementById("tour").addEventListener("click", actionListener, false);
        //document.getElementById("resume").addEventListener("click", actionListener, false);
        //document.getElementById("pause").addEventListener("click", actionListener, false);

        document.getElementById("spherical1").addEventListener("click", actionListener, false);
        document.getElementById("spherical2").addEventListener("click", actionListener, false);
        document.getElementById("spherical3").addEventListener("click", actionListener, false);
        document.getElementById("spherical4").addEventListener("click", actionListener, false);

        document.getElementById("planar1").addEventListener("click", actionListener, false);
        document.getElementById("planar2").addEventListener("click", actionListener, false);
        document.getElementById("planar3").addEventListener("click", actionListener, false);
        document.getElementById("planar4").addEventListener("click", actionListener, false);

        document.getElementById("conical1").addEventListener("click", actionListener, false);
        document.getElementById("conical2").addEventListener("click", actionListener, false);
        document.getElementById("conical3").addEventListener("click", actionListener, false);
        document.getElementById("conical4").addEventListener("click", actionListener, false);
        
        document.getElementById("nodeMenuLink").addEventListener("click", actionListener, false);
        document.getElementById("nodeMenuClose").addEventListener("click", actionListener, false);
        
        document.getElementById("stepSize").addEventListener("change", actionListener, false);
        //document.getElementById("flotTourTest").addEventListener("click", actionListener, false);

        //document.getElementById("").addEventListener("", actionListener, false);
}, false);            


function pausecomp(millis) {
    var date = new Date();
    var curDate = null;

    do { curDate = new Date(); }
    while(curDate-date < millis);
  }

function actionListener(evt)
{   
    if(evt.target == document.getElementById("decreaseFlotExtentButton"))
    {
        changeExtent(DECREASE_EXTENT);
        flotPlot();
    }    
    else if(evt.target == document.getElementById("increaseFlotExtentButton"))
    {
        changeExtent(INCREASE_EXTENT);
        flotPlot();
    }
    else if(evt.target == document.getElementById("waveFunctionCheckbox") ||
            evt.target == document.getElementById("electronDensityCheckbox") ||
            evt.target == document.getElementById("radialProbabilityDistributionCheckbox") ||
            evt.target == document.getElementById("mark50") ||
            evt.target == document.getElementById("mark90") ||
            evt.target == document.getElementById("mark95"))
    {
        flotPlot();
    }
    else if(evt.target == document.getElementById("addPointsButton"))
    {
        // Update the orbital in case the user changed them
        updateOrbitalValues();

        // Get the current points
        numPoints = parseInt(document.getElementById("numPoints").value);

        // Add the new points to the current points
        addPoints(parseInt(document.getElementById("pointsToAddText").value));

        // Put the new points back in the UI
        document.getElementById("numPoints").value = numPoints;

        // Redraw jmol with the new number of points
        if(curIsosurfaceMode != D_POINTILLIST_S || curIsosurfaceMode != D_POINTILLIST_M || curIsosurfaceMode != D_POINTILLIST_L)
            curIsosurfaceMode = D_POINTILLIST_M;

        jmolIsosurfaceChange(curIsosurfaceMode);
    }

    else if(evt.target == document.getElementById("decreaseN"))
    {
        changeNTextboxValue(false);
    }
    else if(evt.target == document.getElementById("increaseN"))
    {
        changeNTextboxValue(true);
    }
    else if(evt.target == document.getElementById("decreaseL"))
    {
        changeLTextboxValue(false);
    }
    else if(evt.target == document.getElementById("increaseL"))
    {
        changeLTextboxValue(true);
    }
    else if(evt.target == document.getElementById("decreaseM"))
    {
        changeMTextboxValue(false);
    }
    else if(evt.target == document.getElementById("increaseM"))
    {
        changeMTextboxValue(true);
    }
    else if(evt.target == document.getElementById("showOrbital"))
    {
        // If the current isosurface mode is one of the types shown by clicking
        // on the flot graph, then the values will be incorrect for a different
        // orbital so fall back to density_pointillist
        if(curIsosurfaceMode == RADIAL_SHELL || (document.getElementById("monteCarloRadioButton").checked && haveOrbitalValuesChanged() && 
          (curIsosurfaceMode == CENTILE_50 || curIsosurfaceMode == CENTILE_90 || curIsosurfaceMode == CENTILE_95)))
        {
           curIsosurfaceMode = D_POINTILLIST_M;
        }
        
        if(haveOrbitalValuesChanged())
        {
            mustMCUpdate = true;
        }

        updateOrbitalValues();

        jmolIsosurfaceChange(curIsosurfaceMode);
    }
    else if(evt.target == document.getElementById("pPointillistParent"))
    {
        alert("Please select a phase pointillist size from the submenu.");
    }
    else if(evt.target == document.getElementById("pPointillistS"))
    {
        jmolIsosurfaceChange(P_POINTILLIST_S);
    }
    else if(evt.target == document.getElementById("pPointillistM"))
    {
        jmolIsosurfaceChange(P_POINTILLIST_M);
    }
    else if(evt.target == document.getElementById("pPointillistL"))
    {
        jmolIsosurfaceChange(P_POINTILLIST_L);
    }
    else if(evt.target == document.getElementById("dPointillistParent"))
    {
        alert("Please select a density pointillist size from the submenu.");
    }
    else if(evt.target == document.getElementById("dPointillistS"))
    {
        jmolIsosurfaceChange(D_POINTILLIST_S);
    }
    else if(evt.target == document.getElementById("dPointillistM"))
    {
        jmolIsosurfaceChange(D_POINTILLIST_M);
    }
    else if(evt.target == document.getElementById("dPointillistL"))
    {
        jmolIsosurfaceChange(D_POINTILLIST_L);
    }
    else if(evt.target == document.getElementById("isosurface"))
    {
        jmolIsosurfaceChange(ISOSURFACE);
    }
    else if(evt.target == document.getElementById("wireframe"))
    {
        jmolIsosurfaceChange(WIREFRAME);
    }
    else if(evt.target == document.getElementById("centileParent"))
    {
        alert("Please select a percentage from the submenu.");
    }
    else if(evt.target == document.getElementById("centile50"))
    {
        jmolIsosurfaceChange(CENTILE_50_M);
    }
    else if(evt.target == document.getElementById("centile90"))
    {
        jmolIsosurfaceChange(CENTILE_90_M);
    }
    else if(evt.target == document.getElementById("centile95"))
    {
        jmolIsosurfaceChange(CENTILE_95_M);
    }
    else if(evt.target == document.getElementById("crossSectionParent"))
    {
        alert("Please select a cross section plane from the submenu.");
    }
    else if(evt.target == document.getElementById("crossSectionXY"))
    {
        jmolIsosurfaceChange(CROSS_SECTION_XY);
    }
    else if(evt.target == document.getElementById("crossSectionXZ"))
    {
        jmolIsosurfaceChange(CROSS_SECTION_XZ);
    }
    else if(evt.target == document.getElementById("crossSectionYZ"))
    {
        jmolIsosurfaceChange(CROSS_SECTION_YZ);
    }
    else if(evt.target == document.getElementById("contourParent"))
    {
        alert("Please select a contour plane from the submenu.");
    }
    else if(evt.target == document.getElementById("contourXY"))
    {
        jmolIsosurfaceChange(CONTOUR_XY);
    }
    else if(evt.target == document.getElementById("contourXZ"))
    {
        jmolIsosurfaceChange(CONTOUR_XZ);
    }
    else if(evt.target == document.getElementById("contourYZ"))
    {
        jmolIsosurfaceChange(CONTOUR_YZ);
    }

    else if(evt.target == document.getElementById("jmolWhiteBg"))
    {
        jmolChangeBgColor("[xFFFFFF]");
    }
    else if(evt.target == document.getElementById("jmolBlackBg"))
    {
        jmolChangeBgColor("[x000000]");
    }
    
    else if(evt.target == document.getElementById("jmolFitZoom"))
    {
        jmolSetToScale(false);
    }
    else if(evt.target == document.getElementById("jmolFitScale"))
    {
        jmolSetToScale(true);
    }

    else if(evt.target == document.getElementById("scaleX"))
    {
        if(document.getElementById("scaleX").checked)
            jmolShowAxesTicks("x");
        else
            jmolHideAxesTicks("x");
    }
    else if(evt.target == document.getElementById("scaleY"))
    {
        if(document.getElementById("scaleY").checked)
            jmolShowAxesTicks("y");
        else
            jmolHideAxesTicks("y");    
    }
    else if(evt.target == document.getElementById("scaleZ"))
    {
        if(document.getElementById("scaleZ").checked)
            jmolShowAxesTicks("z");
        else
            jmolHideAxesTicks("z");
    }
    else if(evt.target == document.getElementById("3d"))
    {
        // 3D options are only applicable for cross-section and contour
        if(curIsosurfaceMode == CROSS_SECTION_XY || curIsosurfaceMode == CROSS_SECTION_XZ || curIsosurfaceMode == CROSS_SECTION_YZ ||
           curIsosurfaceMode == CONTOUR_XY || curIsosurfaceMode == CONTOUR_XZ || curIsosurfaceMode == CONTOUR_YZ)
        {
            (document.getElementById("3d").checked) ? is3D = true : is3D = false;
            jmolIsosurfaceChange(curIsosurfaceMode);
        }
    }
    else if(evt.target == document.getElementById("spin"))
    {
        if(document.getElementById("spin").checked)
            jmolSpin(true, 10, 15, 28);
        else
            jmolSpin(false);
    }
    else if(evt.target == document.getElementById("axes"))
    {
        if(document.getElementById("axes").checked)
        {
            jmolToggleAxes(true);
            jmolSetAxesDiameter();
        }
        else
            jmolToggleAxes(false);
    }

    else if(evt.target == document.getElementById("autoStepSize"))
    {
        document.getElementById("stepSize").disabled = true;
        flotPlot();
    }
    else if(evt.target == document.getElementById("manualStepSize"))
    {
        document.getElementById("stepSize").disabled = false;
        flotPlot();
    }
    else if(evt.target == document.getElementById("stepSize"))
    {
        flotPlot();
    }
    
    else if(evt.target == document.getElementById("reset"))
    {
        resetPage();
    }
    else if(evt.target == document.getElementById("tour"))
    {
        isTourPlaying ? stopTour() : startTour();
    }
    /*else if(evt.target == document.getElementById("resume"))
    {
        tourPaused = false;
        resumeTour();
    }
    else if(evt.target == document.getElementById("pause"))
    {
        isTourPaused = true;
    }*/

    else if(evt.target == document.getElementById("spherical1"))
    {
        (evt.target.checked) ? showSphericalNode(1) : jmolHideSphericalNode(1);
    }
    else if(evt.target == document.getElementById("spherical2"))
    {
        (evt.target.checked) ? showSphericalNode(2) : jmolHideSphericalNode(2);
    }
    else if(evt.target == document.getElementById("spherical3"))
    {
        (evt.target.checked) ? showSphericalNode(3) : jmolHideSphericalNode(3);
    }
    else if(evt.target == document.getElementById("spherical4"))
    {
        (evt.target.checked) ? showSphericalNode(4) : jmolHideSphericalNode(4);
    }

    else if(evt.target == document.getElementById("conical1"))
    {
        (evt.target.checked) ? showConicalNode(1) : jmolHideConicalNode(1);
    }
    else if(evt.target == document.getElementById("conical2"))
    {
        (evt.target.checked) ? showConicalNode(2) : jmolHideConicalNode(2);
    }
    else if(evt.target == document.getElementById("conical3"))
    {
        (evt.target.checked) ? showConicalNode(3) : jmolHideConicalNode(3);
    }
    else if(evt.target == document.getElementById("conical4"))
    {
        (evt.target.checked) ? showConicalNode(4) : jmolHideConicalNode(4);
    }
    
    else if(evt.target == document.getElementById("planar1"))
    {
        (evt.target.checked) ? showPlanarNode(1) : jmolHidePlanarNode(1);
    }
    else if(evt.target == document.getElementById("planar2"))
    {
        (evt.target.checked) ? showPlanarNode(2) : jmolHidePlanarNode(2);
    }
    else if(evt.target == document.getElementById("planar3"))
    {
        (evt.target.checked) ? showPlanarNode(3) : jmolHidePlanarNode(3);
    }
    else if(evt.target == document.getElementById("planar4"))
    {
        (evt.target.checked) ? showPlanarNode(4) : jmolHidePlanarNode(4);
    }
    else if(evt.target == document.getElementById("nodeMenuLink"))
    {
        displayNodeMenu();
    }
    else if(evt.target == document.getElementById("nodeMenuClose"))
    {
        closeNodeMenu();
    }

    else if(evt.target == document.getElementById("monteCarloRadioButton"))
    {
        checkRadialCheckboxes();
        setAutoExtent();
        extent = getExtent(jmolGetN(), jmolGetL());
        numPoints = parseInt(document.getElementById("numPoints").value);
        updateSeed();
        
        // If MC was unchecked just replot flot so MC will be removed from flot
        if(document.getElementById("monteCarloRadioButton").checked == false)
        {
            flotPlot();
        }
        // If not in density pointillist mode, switch to it
        else if((curIsosurfaceMode != D_POINTILLIST_S && curIsosurfaceMode != D_POINTILLIST_M && curIsosurfaceMode != D_POINTILLIST_L) || isPrecomputed())
        {
            jmolIsosurfaceChange(D_POINTILLIST_M);
        // If still in density pointillist and the orbital values have changed, update them and jmol
        }
        else if(haveOrbitalValuesChanged() || mustMCUpdate)
        {
            mustMCUpdate = false;
            updateOrbitalValues();
            jmolIsosurfaceChange(curIsosurfaceMode);
        }
        // If the orbital values are the same, the isosurfaceData is still valid, so just plot flot
        else
        {
        	// always get new (seed) data on toggle
        	updateOrbitalValues();
            jmolIsosurfaceChange(curIsosurfaceMode);
            // ----
            flotPlot();
        }
    }
    else if(evt.target == document.getElementById("theoreticalRadioButton"))
    {
        updateOrbitalValues();
        checkRadialCheckboxes();
        setAutoExtent();
        //extent = getExtent(jmolGetN(), jmolGetL());
        numPoints = parseInt(document.getElementById("numPoints").value);
        updateSeed();
        
        flotPlot(); 
    }
}


function checkRadialCheckboxes()
{
    if(document.getElementById("theoreticalRadioButton").checked)
    {
        document.getElementById("waveFunctionCheckbox").disabled = false;
        document.getElementById("electronDensityCheckbox").disabled = false;
        document.getElementById("radialProbabilityDistributionCheckbox").disabled = false;
        document.getElementById("radialProbabilityDistributionCheckbox").checked = true;

        document.getElementById("mark50").disabled = false;
        document.getElementById("mark90").disabled = false;
        document.getElementById("mark95").disabled = false;
    }
    else if(document.getElementById("monteCarloRadioButton").checked)
    {
        document.getElementById("radialProbabilityDistributionCheckbox").disabled = false;
        document.getElementById("radialProbabilityDistributionCheckbox").checked = true;
    }
    else
    {
        document.getElementById("waveFunctionCheckbox").checked = false;
        document.getElementById("electronDensityCheckbox").checked = false;
        document.getElementById("radialProbabilityDistributionCheckbox").checked = false;


        document.getElementById("waveFunctionCheckbox").disabled = true;
        document.getElementById("electronDensityCheckbox").disabled = true;
        document.getElementById("radialProbabilityDistributionCheckbox").disabled = true;
        
        document.getElementById("mark50").checked = false;
        document.getElementById("mark90").checked = false;
        document.getElementById("mark95").checked = false;

        document.getElementById("mark50").disabled = true;
        document.getElementById("mark90").disabled = true;
        document.getElementById("mark95").disabled = true;
    }

    // Enable/disable add points button and step size controls
    if(document.getElementById("monteCarloRadioButton").checked)
    {
        document.getElementById("addPointsButton").disabled = false;
        document.getElementById("autoStepSize").disabled = false;
        document.getElementById("manualStepSize").disabled = false;
    }
    else
    {
        document.getElementById("addPointsButton").disabled = true;
        document.getElementById("autoStepSize").disabled = true;
        document.getElementById("manualStepSize").disabled = true;
    }
}


function setAutoExtent()
{
    return document.getElementById("flotExtentText").value = getExtent(jmolGetN(), jmolGetL());
}


function setupTransparencySlider(value)
{
    if(isNaN(value))
       value = DEFAULT_TRANSPARENCY_VALUE;

    $("#transparencySlider").slider(
    {
        animate: true,
        value: value,
        min: 1,
        max: 100,
        step: 2,
        slide: function(e, ui)
        {
            jmolChangeTransparencyByID(ui.value);
        }
    });
}


function setupZoomSlider(value)
{
    if(isNaN(value))
        value = DEFAULT_ZOOM_VALUE;

    $("#zoomSlider").slider(
    {
        animate: true,
        value: value,
        min: 10,
        max: 250,
        step: 2,
        slide: function(e, ui)
        {
            jmolChangeZoom(ui.value);
        },
        start: function(e, ui)
        {
            // If the zoom level was changed directly through the applet updated the slider when the
            // user starts sliding it. This will cause a 'jump' in the slider, but without some type of listener
            // or callback to listen for zoom changes implemented in jmol itself, there is no way around this.
            $("#zoomSlider").slider("option", "value", jmolGetZoomValue());    
        }
    });
}

    
function setupSphericalNodeSlider(value)
{
    if(isNaN(value))
       value = DEFAULT_MC_NODE_TRANSPARENCY;

    $("#sphericalSlider").slider(
    {
        animate: true,
        value: value,
        min: 1,
        max: 100,
        step: 2,
        slide: function(e, ui)
        {
            var sphericalTransSelect = document.getElementById("sphericalTransSelect");
            jmolChangeTransparencyByID(ui.value, "spherical" + sphericalTransSelect.options[sphericalTransSelect.selectedIndex].value);
        }
    });
}

function setupConicalNodeSlider(value)
{
    if(isNaN(value))
       value = DEFAULT_MC_NODE_TRANSPARENCY;

    $("#conicalSlider").slider(
    {
        animate: true,
        value: value,
        min: 1,
        max: 100,
        step: 2,
        slide: function(e, ui)
        {
            var conicalTransSelect = document.getElementById("conicalTransSelect");
            jmolChangeTransparencyByID(ui.value, "conical" + conicalTransSelect.options[conicalTransSelect.selectedIndex].value);
        }
    });
}

function setupPlanarNodeSlider(value)
{
    if(isNaN(value))
       value = DEFAULT_MC_NODE_TRANSPARENCY;

    $("#planarSlider").slider(
    {
        animate: true,
        value: value,
        min: 1,
        max: 100,
        step: 2,
        slide: function(e, ui)
        {
            var planarTransSelect = document.getElementById("planarTransSelect");
            jmolChangeTransparencyByID(ui.value, "planar" + planarTransSelect.options[planarTransSelect.selectedIndex].value);
        }
    });
}


function jmolHighlightSphereicalShell(atRadius)
{
    // Only show the shell if MC or Th are active
    if(document.getElementById("monteCarloRadioButton").checked || document.getElementById("radialProbabilityDistributionCheckbox").checked)
    {
        // Prevent flot from calling this function multiple times resulting in
        // "flashing" in jmol
        if(previousRadius == atRadius)
            return;
        
        // Assign the last used radius to a global variable for re-calling the call
        // to jmol outside of this function if the zoom type is changed
        lastRadius = atRadius;

        jmolIsosurfaceChange(RADIAL_SHELL);
    }
}


function changeNTextboxValue(increment)
{
    // Get current N value
    var curValue = parseInt(document.getElementById("orbitalInputN").value);
    
    if(increment)
    {
        // Show error if trying to make N invalid
        if(curValue == MAX_N_MAGNITUDE)
                alert("Only orbitals with n \u2264 5 can be displayed.");
        else
            curValue++;
    }
    else if(!increment)
    {
        // Show error if trying to make N invalid
        if(curValue == 1)
                alert("n must be \u2265 1");
        else
        {
            curValue--;
            
            // Check that the other checkboxes aren't invalid now
            checkLTextboxValue(curValue);
        }
    }

    // Update N
    document.getElementById("orbitalInputN").value = curValue;
}

function changeLTextboxValue(increment)
{
    // Get current L and N values
    var curValue = parseInt(document.getElementById("orbitalInputL").value);
    var curN = parseInt(document.getElementById("orbitalInputN").value);

    if(increment)
    {
        // Show error if attempting to make L invalid
        if(curValue == (curN-1))
            alert("Only orbitals with \u2113 \u2264 n-1 (in this case, " + (curN-1) + ") can be displayed."); 
        // Else, increment L
        else
            curValue++;
    }
    else if(!increment)
    {
        // Show error if attempting to make L invalid
        if(curValue == 0)
            alert("\u2113 must be \u2265 0");
        else
        {
            // Decrement L
            curValue--;
            
            // Check that the other checkboxes aren't invalid now
            checkMTextboxValue();
        }
    }

    // Update L
    document.getElementById("orbitalInputL").value = curValue;
}

function changeMTextboxValue(increment)
{
    // Get current M and L values
    var curValue = parseInt(document.getElementById("orbitalInputM").value);
    var curL = parseInt(document.getElementById("orbitalInputL").value);

    // Show error if attempting to make M invalid
    if((!increment && curValue == -curL) || (increment && curValue == curL))
        alert("Only orbitals with m(\u2113) \u2264 -\u2113 and \u2265 \u2113 (in this case, " + -curL + " and " + curL + ") can be displayed.");
    // Else, increment or decrement M appropriately
    else
    {
        if(increment)
            curValue++;
        else if(!increment)
            curValue--;
    }

    // Update M
    document.getElementById("orbitalInputM").value = curValue;
}


function checkLTextboxValue(curNValue) {
    // L must be >= 0 and <= N
    // Check is this is the case if not, adjust the values
    if(document.getElementById("orbitalInputL").value > (curNValue-1)) {
        document.getElementById("orbitalInputL").value = document.getElementById("orbitalInputL").value-1;
        // If L was changed, M needs checked now as well
        checkMTextboxValue();
    }
}


function checkMTextboxValue() {
    // M is <= l and >= l
    // Check is this is the case if not, adjust the values
    if(document.getElementById("orbitalInputM").value >= document.getElementById("orbitalInputL").value)
        document.getElementById("orbitalInputM").value = document.getElementById("orbitalInputM").value-1;
    else if(document.getElementById("orbitalInputM").value <= -document.getElementById("orbitalInputL").value)
        document.getElementById("orbitalInputM").value = parseInt(document.getElementById("orbitalInputM").value)+1;
}


function isOrbitalInputValid()
{
    // Jmol holds javascript execution while it is performing calculations. This will allow
    // a user to bypass the checks on the orbital N, L, and M values and obtain an invalid value
    // in some or all of the textboxes.
    // Perform a final check to make sure that everything is still valid before displaying another orbital
    var n = parseInt(document.getElementById("orbitalInputN").value);
    var l = parseInt(document.getElementById("orbitalInputL").value);
    var m = parseInt(document.getElementById("orbitalInputM").value);
    
    return ((n > 0 && n <= MAX_N_MAGNITUDE) && (l >= 0 && l < n) && (Math.abs(m) <= l));
}


function jmolIsosurfaceChange(isosurfaceMode)
{

    // Check if an argument was given
    if(isNaN(isosurfaceMode))
        isosurfaceMode = curIsosurfaceMode;

    // Update the orbital numbers again for good measure
    updateOrbitalValues();

    // Set the name label
    updateOrbitalNameLabel();
    
    // Update the nodes label, the nodes checkboxes, and the nodes transparency menu
    updateNodesLabel();
    setNodesCheckboxes();
    setNodeMenu();

    // Invalidate previousMarker
    previousMarker = null;

    // Clear all isosurfaces in advance of the new isosurface
    jmolDeleteAllIsosurfaces();

    // Update curIsosurfaceMode
    curIsosurfaceMode = isosurfaceMode;

    if(isosurfaceMode == P_POINTILLIST_S)
    {
        setIsosurfaceMenuLabel("Phase Pointillist S");
        if(isPrecomputed() && !document.getElementById("monteCarloRadioButton").checked) {
            jmolSetDotScale(SMALL_POINTILLIST_DOTS);
            jmolLoadModel(MODELS_LOCATION + jmolGetN() + jmolGetL() + jmolGetM() + "_phase.jvxl");
        } else {
            jmolShowPointillist(SMALL_POINTILLIST_DOTS, numPoints, DEFAULT_MC_CUTOFF, DEFAULT_MC_RESOLUTION);
        }
    }
    else if(isosurfaceMode == P_POINTILLIST_M)
    {    
        setIsosurfaceMenuLabel("Phase Pointillist M");
        if(isPrecomputed() && !document.getElementById("monteCarloRadioButton").checked) {
            jmolSetDotScale(MEDIUM_POINTILLIST_DOTS);
            jmolLoadModel(MODELS_LOCATION + jmolGetN() + jmolGetL() + jmolGetM() + "_phase.jvxl");
        } else {
            jmolShowPointillist(MEDIUM_POINTILLIST_DOTS, numPoints, DEFAULT_MC_CUTOFF, DEFAULT_MC_RESOLUTION);
        }
    }
    else if(isosurfaceMode == P_POINTILLIST_L)
    {
    
    alert("ok ppoint")
        setIsosurfaceMenuLabel("Phase Pointillist L");
        if(isPrecomputed() && !document.getElementById("monteCarloRadioButton").checked) {
            jmolSetDotScale(LARGE_POINTILLIST_DOTS);
            jmolLoadModel(MODELS_LOCATION + jmolGetN() + jmolGetL() + jmolGetM() + "_phase.jvxl");
        } else {
            jmolShowPointillist(LARGE_POINTILLIST_DOTS, numPoints, DEFAULT_MC_CUTOFF, DEFAULT_MC_RESOLUTION);
        }
    }
    else if(isosurfaceMode == D_POINTILLIST_S)
    {
        setIsosurfaceMenuLabel("Density Pointillist S");
        if(isPrecomputed() && !document.getElementById("monteCarloRadioButton").checked) {
            jmolSetDotScale(SMALL_POINTILLIST_DOTS);
            jmolLoadModel(MODELS_LOCATION + jmolGetN() + jmolGetL() + jmolGetM() + "_density.jvxl");
        } else {
            jmolShowDensityPointillist(SMALL_POINTILLIST_DOTS, numPoints, seed);
        }
    }
    else if(isosurfaceMode == D_POINTILLIST_M)
    {
        setIsosurfaceMenuLabel("Density Pointillist M");
        if(isPrecomputed() && !document.getElementById("monteCarloRadioButton").checked) {
            jmolSetDotScale(MEDIUM_POINTILLIST_DOTS);
            jmolLoadModel(MODELS_LOCATION + jmolGetN() + jmolGetL() + jmolGetM() + "_density.jvxl");
        } else {
            jmolShowDensityPointillist(MEDIUM_POINTILLIST_DOTS, numPoints, seed);
        }
    }
    else if(isosurfaceMode == D_POINTILLIST_L)
    {
        setIsosurfaceMenuLabel("Density Pointillist L");
        if(isPrecomputed() && !document.getElementById("monteCarloRadioButton").checked) {
            jmolSetDotScale(LARGE_POINTILLIST_DOTS);
            jmolLoadModel(MODELS_LOCATION + jmolGetN() + jmolGetL() + jmolGetM() + "_density.jvxl");
        } else {
            jmolShowDensityPointillist(LARGE_POINTILLIST_DOTS, numPoints, seed);
        }
    }
    else if(isosurfaceMode == ISOSURFACE)
    {
        setIsosurfaceMenuLabel("Isosurface");
        jmolShowSolidIsosurface(DEFAULT_MC_CUTOFF, DEFAULT_MC_RESOLUTION);
    }
    else if(isosurfaceMode == WIREFRAME)
    {
        setIsosurfaceMenuLabel("Wireframe");
        jmolShowWireframe(DEFAULT_MC_CUTOFF, getWireframeResolution());
    }
    else if(isosurfaceMode == CENTILE_50 || isosurfaceMode == CENTILE_50_M)
    {
        setIsosurfaceMenuLabel("Centile Boundary 50%");
        jmolShowCentileBoundary(DEFAULT_MC_CUTOFF, getCentileCutoff(7), getWireframeResolution());
    }
    else if(isosurfaceMode == CENTILE_90 || isosurfaceMode == CENTILE_90_M)
    {
        setIsosurfaceMenuLabel("Centile Boundary 90%");
        jmolShowCentileBoundary(DEFAULT_MC_CUTOFF, getCentileCutoff(8), getWireframeResolution());
    }
    else if(isosurfaceMode == CENTILE_95 || isosurfaceMode == CENTILE_95_M)
    {
        setIsosurfaceMenuLabel("Centile Boundary 95%");
        jmolShowCentileBoundary(DEFAULT_MC_CUTOFF, getCentileCutoff(9), getWireframeResolution());
    }
    else if(isosurfaceMode == CROSS_SECTION_XY)
    {
        setIsosurfaceMenuLabel("Cross-Section XY");
        (!is3D) ? jmolShowCrossSection("XY") : jmolShowCrossSection3D("XY");
    }
    else if(isosurfaceMode == CROSS_SECTION_XZ)
    {
        setIsosurfaceMenuLabel("Cross-Section XZ");
        (!is3D) ? jmolShowCrossSection("XZ") : jmolShowCrossSection3D("XZ");
    }
    else if(isosurfaceMode == CROSS_SECTION_YZ)
    {
        setIsosurfaceMenuLabel("Cross-Section YZ");
        (!is3D) ? jmolShowCrossSection("YZ") : jmolShowCrossSection3D("YZ");
    }
    else if(isosurfaceMode == CONTOUR_XY)
    {
        setIsosurfaceMenuLabel("Contour XY");
        (!is3D) ? jmolShowContour("XY") : jmolShowContour3D("XY");
    }
    else if(isosurfaceMode == CONTOUR_XZ)
    {
        setIsosurfaceMenuLabel("Contour XZ");
        (!is3D) ? jmolShowContour("XZ") : jmolShowContour3D("XZ");
    }
    else if(isosurfaceMode == CONTOUR_YZ)
    {
        setIsosurfaceMenuLabel("Contour YZ");
        (!is3D) ? jmolShowContour("YZ") : jmolShowContour3D("YZ");
    }
    else if(isosurfaceMode == RADIAL_SHELL)
    {
        setIsosurfaceMenuLabel("Radial Shell");
        jmolShowIsosurfaceSkin(DEFAULT_MC_CUTOFF);
        jmolShowIsosurfaceShell(numPoints, lastRadius);
    }
    
    // Show the calculating menu after given amount of a time (in ms)
    showCalcMenu(800);

    // Set the new orbital's transparency to whatever the old one was at
    jmolChangeTransparencyByID($("#transparencySlider").slider("value"));

    // Zoom levels are set independly of the zoom slider so just reset it
    setupZoomSlider();

    // Determine if MC needs disabled and update Flot
    if(document.getElementById("monteCarloRadioButton").checked)
    {
        if(curIsosurfaceMode != D_POINTILLIST_S && curIsosurfaceMode != D_POINTILLIST_M && curIsosurfaceMode != D_POINTILLIST_L &&
           curIsosurfaceMode != RADIAL_SHELL && curIsosurfaceMode != CENTILE_50 && curIsosurfaceMode != CENTILE_90 && curIsosurfaceMode != CENTILE_95)
        {
            document.getElementById("monteCarloRadioButton").checked = false;
            checkRadialCheckboxes();

            setAutoExtent();
            //extent = getExtent(jmolGetN(), jmolGetL());
            flotPlot();
        }
    }
    else if(document.getElementById("theoreticalRadioButton").checked)
    {
        setAutoExtent();
        //extent = getExtent(jmolGetN(), jmolGetL());
        flotPlot();
    }

    // Disable the 3D checkbox if not in cross-section or contour modes anymore
    if(curIsosurfaceMode == CROSS_SECTION_XY || curIsosurfaceMode == CROSS_SECTION_XZ || curIsosurfaceMode == CROSS_SECTION_YZ ||
       curIsosurfaceMode == CONTOUR_XY || curIsosurfaceMode == CONTOUR_XZ || curIsosurfaceMode == CONTOUR_YZ)
    {
        document.getElementById("3d").disabled = false;
    }
    else
    {
        document.getElementById("3d").checked = false;
        document.getElementById("3d").disabled = true;
    }
}


function postJmolCalc()
{
   // The calculation is done so cancel the timer so the menu isn't shown we finished
   // before the timeout period is over
   clearTimeout(calcTimer);
   document.getElementById("calcMenu").style.display = "none";

   // We can only get the isosurface data needed for MC once jmol is done calculating it.
  setAutoExtent();
  //extent = getExtent(jmolGetN(), jmolGetL());

  // If radial shell or the centile boundaries were selected by clicking on the flot graph, it's safe to not
  // update the isosurface data
  if(curIsosurfaceMode != RADIAL_SHELL && curIsosurfaceMode != CENTILE_50 && curIsosurfaceMode != CENTILE_90 && curIsosurfaceMode != CENTILE_95)
      setIsosurfaceData();
   // Thus, if MC is active, pull the data and then plot Flot
   if(document.getElementById("monteCarloRadioButton").checked)
    {

        flotPlot();
    }
}


function showCalcMenu(waitTime)
{
    if(!isNaN(waitTime))
        calcTimer = setTimeout("document.getElementById(\"calcMenu\").style.display = \"block\"", waitTime);
}


function startTour()
{
	// Reset jmol
	jmolReset();
	
	// Reset flot
	displayEmptyFlot(flotPlaceHolder);

    // For now, we're hard coding the tour at 3 2 0. Set the values as such.
    // This block of code is temporary until the tour can accomodate any orbital.
    document.getElementById("orbitalInputN").value = "3";
    document.getElementById("orbitalInputL").value = "2";
    document.getElementById("orbitalInputM").value = "0";
    
    updateOrbitalValues();
    updateOrbitalNameLabel();

    jmolSetIntVariable("tourN", 3);
    jmolSetIntVariable("tourL", 2);
    jmolSetIntVariable("tourM", 0);
    jmolSetIntVariable("tourAxesScale", 1);

    // The extent should be the same for orbital 1 0 * as long as 3 2 0 is the hard coded
    // orbital for the tour
    setAutoExtent();
    extent = getExtent(1, 0);
    // End temp code


    // Change the tour button to a stop tour button
    isTourPlaying = true;
    document.getElementById("tour").value = "Stop Tour";

    // Run tour script
    jmolRunScript(TOUR_LOCATION);
}


function stopTour() {
    // This will only stop the script. We still need to reset the page.
    jmolStopAllScripts();

    // Let the tour finished function take care of resetting the page
    tourFinished();
}


function tourFinished() {
    // Reset the tour button
    document.getElementById("tour").value = "Tour";

    isTourPlaying = false;
	
	// Reset everything else
	resetPage();
}


function setIsosurfaceData()
{
    isosurfaceData = jmolGetIsosurfaceInfo();
    isReady = true;
}


function setIsosurfaceMenuLabel(label)
{
    document.getElementById("curIsosurfaceType").innerHTML = label;
}

function updateOrbitalNameLabel()

{
    document.getElementById("orbitalName").innerHTML = jmolGetOrbitalName();
}


function updateNodesLabel()
{
    document.getElementById("numNodes").innerHTML = jmolGetN() - 1;
}


function setNodesCheckboxes()
{
    // Get number of each type of node
    var numSpherical = getNumSphericalNodes();
    var numConical = getNumConicalNodes();
    var numPlanar = getNumPlanarNodes();
    
    // Disable everything to reset from previous orbital
    resetNodeCheckboxes();
    
    // Enable the correct checkboxes for the current orbital
    for(var i=1; i<=numSpherical; i++)
        document.getElementById("spherical" + i).disabled = false;
    for(var i=1; i<=numConical; i++)
        document.getElementById("conical" + i).disabled = false;
    for(var i=1; i<=numPlanar; i++)
        document.getElementById("planar" + i).disabled = false;
}


function resetNodeCheckboxes()
{
    // Loop through each checkbox and uncheck then disable each one
    // Note that call to unchecked must be called before disable as
    // disabled checkboxes are immutable
    for(var i=1; i<=4; i++)
    {
        document.getElementById("spherical" + i).checked = false;
        document.getElementById("spherical" + i).disabled = true;
        document.getElementById("conical" + i).checked = false;
        document.getElementById("conical" + i).disabled = true;
        document.getElementById("planar" + i).checked = false;
        document.getElementById("planar" + i).disabled = true;
    }
}


function getNumSphericalNodes()
{
    return jmolGetN() - jmolGetL() - 1;
}


function getNumConicalNodes()
{
    return jmolGetL() - Math.abs(jmolGetM());
}


function getNumPlanarNodes()
{
    return Math.abs(jmolGetM());
}


function haveOrbitalValuesChanged()
{
    n = document.getElementById("orbitalInputN").value;
    l = document.getElementById("orbitalInputL").value;
    m = document.getElementById("orbitalInputM").value;
    
    return (jmolGetN() != n || jmolGetL() != l || jmolGetM() != m);
}


function updateOrbitalValues(n, l, m)
{
    if(isNaN(n))
        n = document.getElementById("orbitalInputN").value;
    if(isNaN(l))
        l = document.getElementById("orbitalInputL").value;
    if(isNaN(m))
        m = document.getElementById("orbitalInputM").value;

    // Perform a final check to verify orbital inputs
    if(!isOrbitalInputValid()) {
        // Show error
        alert("Error: The orbital values n, \u2113, or m are invalid. Resetting them to 1 0 0 respectively.");

        // Revert textboxes to default N, L, and M
        document.getElementById("orbitalInputN").value = DEFAULT_N;
        document.getElementById("orbitalInputL").value = DEFAULT_L;
        document.getElementById("orbitalInputM").value = DEFAULT_M;

        // Update local vars
        n = DEFAULT_N;
        l = DEFAULT_L;
        m = DEFAULT_M;
    }
        
    // Update jmol with the new orbital values
    jmolSetOrbitalValues(n, l, m, DEFAULT_Z);
}


function showSphericalNode(numNode)
{
    // First number cooresponds to N value, second number is L value.
    var radiiMap = {"20": [1.06],
                    "30": [1.01, 3.76], "31": [3.18],
                    "40": [1, 3.5, 8.21], "41": [2.93, 7.66], "42": [6.35],
                    "50": [0.99, 3.41, 7.58, 14.49], "51": [2.84, 7.03, 13.95], "52": [5.76, 12.76], "53": [10.58]}

    var radii = radiiMap["" + jmolGetN() + jmolGetL()];

    jmolShowSphericalNode(radii[numNode-1], numNode, DEFAULT_MC_NODE_TRANSPARENCY, DEFAULT_MC_NODE_COLOR);
}


function showConicalNode(numNode)
{
    // First number of key is L value. Second number is M value.
    var anglesMap = {"10": ["0.0"], "21": ["0.0"], "32": ["0.0"], "43": ["0.0"],
                     "20": ["0.715"], "31": ["0.504"], "42": ["0.415"],
                     "30": ["1.229", "0.0"], "41": ["0.851", "0.0"],
                     "40": ["0.35", "1.172"]};
                  
    var angles = anglesMap["" + jmolGetL() + Math.abs(jmolGetM())];
    var c;

    // If first or second node, display cone. If 3rd, display plane
    if(numNode <= 2)
        c = angles[0];
    else if(numNode == 3 && angles.length > 1)
        c = angles[1];

    // If the second node, use the negative value
    if(numNode == 2)
        c = -c;

    jmolShowConicalNode(c, numNode, DEFAULT_MC_NODE_TRANSPARENCY, DEFAULT_MC_NODE_COLOR);

    // There's only one orbital with 4 nodes (L=4, M=0). If this is it, change the c to the second value
    if(c != "0.0" && (numNode == 3 || numNode == 4) && angles.length > 1) {
        c = angles[1];
        
        // Fourth node should use then negative value
        if(numNode == 4)
            c = -c;

        jmolShowConicalNode(c, numNode, DEFAULT_MC_NODE_TRANSPARENCY, DEFAULT_MC_NODE_COLOR);
    }
}


function showPlanarNode(numNode)
{
    var angles = {"-1": ["(0, 90, 90)"],
                      "1": ["(0, 0, 90)"],
                     "-2": ["(0, 0, 90)", "(0, 90, 90)"],
                      "2": ["(0, 45, 90)", "(0, -45, 90)"],
                     "-3": ["(0, 30, 90)", "(0, -30, 90)", "(0, 90, 90)"],
                      "3": ["(0, 0, 90)", "(0, 60, 90)", "(0, -60, 90)"],
                     "-4": ["(0, 0, 90)", "(0, 45, 90)", "(0, 90, 90)", "(0, -45, 90)"],
                      "4": ["(0, 22.5, 90)", "(0, -22.5, 90)", "(0, 67.5, 90)", "(0, -67.5, 90)"]};
    
    jmolShowPlanarNode(angles[jmolGetM()][numNode-1], numNode, DEFAULT_MC_NODE_TRANSPARENCY, DEFAULT_MC_NODE_COLOR);
}


function getWireframeResolution()
{
    // Each n value requires a different resolution for it display properly and
    // within a reasonable amount of time
    var resolutions = {"1": 5, "2": 2.5, "3": 1.1, "4": 0.5, "5": 0.6}
    return resolutions[jmolGetN()];
}


function isPrecomputed()
{
    // Get the type of the current orbital
    var type;
    if(curIsosurfaceMode == P_POINTILLIST_S || curIsosurfaceMode == P_POINTILLIST_M || curIsosurfaceMode == P_POINTILLIST_L)
        type = "phase";
    else if(curIsosurfaceMode == D_POINTILLIST_S || curIsosurfaceMode == D_POINTILLIST_M || curIsosurfaceMode == D_POINTILLIST_L)
        type = "density";
    else
        return 0;

    var precompOrbitals = ["54-4_density", "54-3_density", "54-2_density", "54-1_density", 
                           "544_density", "543_density", "542_density", "541_density",
                           "54-4_phase", "54-3_phase", "54-2_phase", "54-1_phase", 
                           "544_phase", "543_phase", "542_phase", "541_phase"];

    var orbital = jmolGetN() + jmolGetL() + jmolGetM() + "_" + type;

    // Check if the orbital is in the precomputed orbitals array
    for(var i=0; i<precompOrbitals.length; i++) {
        if(orbital == precompOrbitals[i])
            return 1;
    }

    return 0;
}


function resetPage()
{
    // The scale property of jmol should be set back to the zoom
    // as that is the default BEFORE resetting jmol and the default
    // orbital should be updated so jmol_scripting resets to the 
    // proper, default zoom level at N = 1
    updateOrbitalValues(DEFAULT_N, DEFAULT_L, DEFAULT_M);
    jmolSetToScale(false);
    
    // Reset jmol
    jmolReset(true);

    // Reset Flot
    displayEmptyFlot(flotPlaceHolder);

    // Reset orbital text fields and labels
    document.getElementById("orbitalName").innerHTML = "N/A";
    document.getElementById("numNodes").innerHTML = "N/A";

    // Reset curIsosurface menu
    setIsosurfaceMenuLabel("Display Mode");

    // Reset orbital numbers inputs
    document.getElementById("orbitalInputN").value = DEFAULT_N;
    document.getElementById("orbitalInputL").value = DEFAULT_L;
    document.getElementById("orbitalInputM").value = DEFAULT_M;

    // Reset nodes
    resetNodeCheckboxes();

    // Reset scales, spin, and axes checkboxes
    document.getElementById("scaleX").checked = false;
    document.getElementById("scaleY").checked = false;
    document.getElementById("scaleZ").checked = false;

    // Reset zoom
    document.getElementById("jmolFitZoom").checked = true;
    document.getElementById("jmolFitScale").checked = false;

    // Reset MC options
    document.getElementById("monteCarloRadioButton").checked = false;
    document.getElementById("theoreticalRadioButton").checked = false;

    // Reset radial plots options
    checkRadialCheckboxes();

    // Reset extent
    document.getElementById("flotExtentText").value = 10;

    // Reset r max and average
    document.getElementById("rMaxMonteCarlo").innerHTML = "";
    document.getElementById("rMaxTH").innerHTML = "";
    document.getElementById("rAvgMC").innerHTML = "";
    document.getElementById("rAvgTH").innerHTML = "";

    // Reset the sliders
    setupTransparencySlider();
    setupZoomSlider();
    setupSphericalNodeSlider();
    setupConicalNodeSlider();
    setupPlanarNodeSlider();

    // Reset background radio buttons
    document.getElementById("jmolWhiteBg").checked = true;
    document.getElementById("jmolBlackBg").checked = false;

    // Reset step size
    document.getElementById("autoStepSize").checked = true;
    document.getElementById("manualStepSize").checked = false;
    document.getElementById("stepSize").value = 0.05;

    // Reset points
    document.getElementById("numPoints").value = DEFAULT_NUM_POINTS;
    document.getElementById("pointsToAddText").value = DEFAULT_ADD_NUM_POINTS;
}


function playSoundFile(file) {
    //alert("Play sound file: " + param);
}



// Flot related functions

function getCentileCutoff(percentCode)
{
    // Translate the percent code to the percent code used in the maps below
    // 7, 10, 13 = 50%,
    // 8, 11, 14 = 90%,
    // 9, 12, 15 = 95%
    var percent = -1;
    switch(percentCode) {
        case 7:
        case 10:
        case 13:
            percent = "5";
            break;
        case 8:
        case 11:
        case 14:
            percent = "0";
            break;
        case 9:
        case 12:
        case 15:
            percent = "9";
            break;
        default:
            return -1;
    }

    // In the map below, the first number is the N value, the second is L, third is M, and the fourth is the percent with 5 corresponding to 50%, 0 90%, and 9 95%.
    var cutoffs1 = {"1005": 0.1506, "1000": 0.03983, "1009": 0.02483,
                   "2005": 0.02037, "2000": 0.007458, "2009": 0.004911,
                   "2105": 0.03162, "2100": 0.009518, "2109": 0.006036,
                   "3005": 0.006237, "3000": 0.002669, "3009": 0.00181,
                   "3105": 0.008404, "3100": 0.00305, "3109": 0.002036,
                   "3205": 0.01065, "3200": 0.003621, "3209": 0.002386,
                   "32-15": 0.01008, "32-10": 0.003502, "32-19": 0.00229,
                   "4005": 0.002681, "4000": 0.001264, "4009": 0.000872,
                   "4105": 0.003484, "4100": 0.001381, "4109": 0.0938,
                   "4205": 0.00376, "4200": 0.001484, "4209": 0.001019,
                   "42-15": 0.003588, "42-10": 0.001452, "42-19": 0.000993,
                   "4305": 0.004896, "4300": 0.001796, "4309": 0.001185,
                   "43-15": 0.004733, "43-10": 0.001752, "43-19": 0.001171,
                   "43-25": 0.0047, "43-20": 0.001741, "43-29": 0.00116,
                   "43-35": 0.004957, "43-30": 0.001809, "43-39": 0.001194,
                   "5005": 0.001398, "5000": 0.000689, "5009": 0.000487,
                   "5105": 0.001782, "5100": 0.000738, "5109": 0.000511,
                   "5205": 0.001827, "5200": 0.000769, "5209": 0.000539,
                   "52-15": 0.001759, "52-10": 0.000759, "52-19": 0.000528,
                   "5305": 0.002046, "5300": 0.000842, "5309": 0.000582,
                   "53-15": 0.00201, "53-10": 0.000858, "53-19": 0.000576,
                   "53-25": 0.002005, "53-20": 0.000834, "53-29": 0.000574,
                   "53-35": 0.002106, "53-30": 0.000866, "53-39": 0.000595,
                   "5405": 0.002627, "5400": 0.001024, "5409": 0.000669,
                   "54-15": 0.002605, "54-10": 0.001003, "54-19": 0.000678,
                   "54-25": 0.002618, "54-20": 0.001011, "54-29": 0.000684,
                   "54-35": 0.002738, "54-30": 0.001046, "54-39": 0.000703,
                   "54-45": 0.002842, "54-40": 0.001073, "54-49": 0.000714};

    var key = jmolGetN() + jmolGetL() + jmolGetM() + percent;

    // Try to find the key in cutoffs1. If it is exists, return; we're done. If not, try fall back to cutoffs2 where it must exist. 
    var cutoff = cutoffs1[key];

    if(!(typeof cutoff === "undefined"))
        return cutoff;

    // In the map below, the first number is the N value, the second is L, and the third is M
    var cutoffs2 = {"21-1": "210",  "211": "210",
                    "31-1": "310",  "311": "310",
                    "321": "32-1",  "32-2": "32-1",     "322": "32-1",
                    "41-1": "410",  "411": "410",
                    "421": "42-1",  "42-2": "42-1",     "422": "42-1",
                    "431": "43-1",
                    "432": "43-2",
                    "433": "43-3",
                    "51-1": "510",  "511": "510",
                    "521": "52-1",  "52-2": "52-1",     "522": "52-1",
                    "531": "53-1",
                    "532": "53-2",
                    "533": "53-3",
                    "541": "54-1",
                    "542": "54-2",
                    "543": "54-3",
                    "544": "54-4"};

    key = cutoffs2[jmolGetN() + jmolGetL() + jmolGetM()] + percent;

    cutoff = cutoffs1[key]

    return (!(typeof cutoff === "undefined")) ? cutoff : -1;
}


// flotPlot and related functions.  flotPlot determines which (theoretical or jMol) to graph.
//  The coresponding method is then called.


function flotPlot()
// POST: html inputs are checked and the appropriate data is plotted in flot
{   
    var n;                                      // principle quantum number
    var l;                                      // azimuthal qunatum number
    var monteCarlo;                             // status of MC radio button
    var theoretical;                            // status of TH radio button
    var simultaneous;                           // status of Both radio button
    var theoWaveFunction;                       // status of R checkbox
    var theoelectronDensity;                    // status of R^2 checkbox
    var theoRradialProbability;                 // status of (r^2)*(R^2) checkbox
    var mark50;                                 // status of Mark At: 50% checkbox
    var mark90;                                 // status of Mark At: 90% checkbox
    var mark95;                                 // status of Mark At: 95% checkbox
    var manualStepSize;                         // the value (if not null) of the manual stepSize
    var extent;

    previousMarker = null;
    // get status of all boxes concerning flot plots
    extent = parseInt(document.getElementById("flotExtentText").value);

    monteCarlo = document.getElementById("monteCarloRadioButton").checked;
    theoretical = document.getElementById("theoreticalRadioButton").checked;
    simultaneous = monteCarlo && theoretical;

    n = jmolGetN();
    l = jmolGetL();

    theoWaveFunction = document.getElementById("waveFunctionCheckbox").checked;
    theoelectronDensity = document.getElementById("electronDensityCheckbox").checked;
    theoRradialProbability = document.getElementById("radialProbabilityDistributionCheckbox").checked;

    mark50 = document.getElementById("mark50").checked;
    mark90 = document.getElementById("mark90").checked;
    mark95 = document.getElementById("mark95").checked;
    
    manualStepSize = document.getElementById("manualStepSize").checked ? parseFloat(document.getElementById("stepSize").value) : null;

    // displayFlot(monteCarloData, mcBarwidth, waveFunction, electronDensity, radialProbabilityDistribution, marker50, marker90, marker95, extent, flotPlaceHolder)
    displayFlot( ( isReady && (monteCarlo || simultaneous) ? getJmolData(n, l, seed, numPoints, isosurfaceData.vertices, manualStepSize) : [null] ), getMCStep(),
                 (theoWaveFunction && (theoretical || simultaneous) ? getWaveFunction(n, l) : [null, null]),
                 (theoelectronDensity && (theoretical || simultaneous) ? getElectronDensity(n, l) : [null, null]),
                 (theoRradialProbability && (theoretical || simultaneous) ? getRadialProbabilityDistribution(n, l) : [null, null]),
                 ((mark50 && theoRradialProbability && (theoretical || simultaneous)) ? getr2R2Marker50() : [null]),
                 ((mark90 && theoRradialProbability && (theoretical || simultaneous)) ? getr2R2Marker90() : [null]),
                 ((mark95 && theoRradialProbability && (theoretical || simultaneous)) ? getr2R2Marker95() : [null]),
                 ((mark50 && theoelectronDensity && (theoretical || simultaneous)) ? getr2Marker50() : [null]),
                 ((mark90 && theoelectronDensity && (theoretical || simultaneous)) ? getr2Marker90() : [null]),
                 ((mark95 && theoelectronDensity && (theoretical || simultaneous)) ? getr2Marker95() : [null]),
                 ((mark50 && theoWaveFunction && (theoretical || simultaneous)) ? getrMarker50() : [null]),
                 ((mark90 && theoWaveFunction && (theoretical || simultaneous)) ? getrMarker90() : [null]),
                 ((mark95 && theoWaveFunction && (theoretical || simultaneous)) ? getrMarker95() : [null]),
                 extent, flotPlaceHolder);
    
    var value = montecarloGetMostProbablyRadius();
    if(!(typeof value === "undefined"))
        document.getElementById("rMaxMonteCarlo").innerHTML = value.toFixed(1);
    else
        document.getElementById("rMaxMonteCarlo").innerHTML = "N/A";

    value = montecarloGetAverageRadius();
    if(!(typeof value === "undefined"))
        document.getElementById("rAvgMC").innerHTML = value.toFixed(1);
    else
        document.getElementById("rAvgMC").innerHTML = "N/A";

    value = schrodingerGetMostProbablyRadius();
    if(!(typeof value === "undefined"))
        document.getElementById("rMaxTH").innerHTML = value.toFixed(1);
    else
        document.getElementById("rMaxTH").innerHTML = "N/A";

    value = schrodingerGetAverageRadius();
    if(!(typeof value === "undefined"))
        document.getElementById("rAvgTH").innerHTML = value.toFixed(1);
    else
        document.getElementById("rAvgTH").innerHTML = "N/A";

}


function displayEmptyFlot(flotPlaceHolder)
{    
    $.plot($(flotPlaceHolder), [{}])
}


function displayFlot(monteCarloData, mcBarwidth, waveFunction, electronDensity, radialProbabilityDistribution, r2R2marker50, r2R2marker90, r2R2marker95,
                     r2marker50, r2marker90, r2marker95, rmarker50, rmarker90, rmarker95, extent, flotPlaceHolder)
{
    var plotCounter = 0;
    var previousPoint = null;
    var markerString;
    
    if(!(typeof mcBarwidth === "undefined"))
       document.getElementById("stepSize").value = mcBarwidth;
           
    var plot = $.plot($(flotPlaceHolder), [
        {
            data: monteCarloData,
            bars:
            {
                show: true,
                barWidth : mcBarwidth,
                fill: true
            }
        },
        {
            data: waveFunction[0],
            color: 1,
            lines:
            {
                show: true,
                fill: true
            } 
        },
        {
            data: waveFunction[1],
            color: 2,
            lines:
            {
                show: true,
                fill: true
            }
        },
        {
            data: electronDensity[0],
            color: 3,
            lines:
            {
                show: true,
                fill: true
            }
        },
        {
            data: electronDensity[1],
            color: 0,
            lines:
            {
                show: true,
                fill: true
            } 
        },
        {
            data: radialProbabilityDistribution[0],
            color: 4,
            lines:
            {
                show: true,
                fill: true
            }
        },
        {
            data: radialProbabilityDistribution[1],
            color: 3,
            lines:
            {
                show: true,
                fill: true
            }
        },
        {
            data: [r2R2marker50],
            color: 1,
            points:
            {
                show: true
            }
        },
        {
            data: [r2R2marker90],
            color: 1,
            points:
            {
                show: true
            }
        },
        {
            data: [r2R2marker95],
            color: 1,
            points:
            {
                show: true
            }
        },
        {
            data: [r2marker50],
            color: 1,
            points:
            {
                show: true
            }
        },
        {
            data: [r2marker90],
            color: 1,
            points:
            {
                show: true
            }
        },
        {
            data: [r2marker95],
            color: 1,
            points:
            {
                show: true
            }
        },
        {
            data: [rmarker50],
            color: 1,
            points:
            {
                show: true
            }
        },
        {
            data: [rmarker90],
            color: 1,
            points:
            {
                show: true
            }
        },
        {
            data: [rmarker95],
            color: 1,
            points:
            {
                show: true
            }
        }],
        {
            crosshair:
            {
                mode: "x"
            },
            grid:
            {
                clickable: true,
                hoverable: true
            },
            xaxis:
            {
                min: 0,
                max: extent
            }
        }
    );

    $(flotPlaceHolder).bind("plotclick",  function (event, pos, item)
        {
            
            if (item && (item.seriesIndex >= 7 && item.seriesIndex <= 15) )
            {
                switch(item.seriesIndex) {
                    case 7:
                    case 10:
                    case 13:
                       jmolIsosurfaceChange(CENTILE_50);
                       break;
                    case 8:
                    case 11:
                    case 14:
                       jmolIsosurfaceChange(CENTILE_90);
                       break;
                    case 9:
                    case 12:
                    case 15:
                       jmolIsosurfaceChange(CENTILE_95);
                    default:
                }

                previousMarker = dataIndex; 
            }
            else
            {
                jmolHighlightSphereicalShell(pos.x);
                previousRadius = pos.x;
            }
         }
    );
    
    
    coords = plot.offset();
    isDisplayed = false;
    
    $(flotPlaceHolder).bind("plothover", function (event, pos, item)
        {        
            //showTooltip(pos.pageX, pos.pageY, temp);
            //t = $(flotPlaceHolder).getCanvas();
            
            //h = getAxes();
            if (item && (item.seriesIndex >= 7 && item.seriesIndex <= 15) )
            {
                if (previousPoint != item.dataIndex)
                {
                    previousPoint = item.dataIndex;
                    
                    $("#markertooltip").remove();
                    var x = item.datapoint[0].toFixed(2),
                        y = item.datapoint[1].toFixed(2);
                        
                    if (item.seriesIndex == 7 || item.seriesIndex == 10 || item.seriesIndex == 13)
                        markerString = "click for 50% boundary surface of total electron density ";
                    else if (item.seriesIndex == 8 || item.seriesIndex == 11 || item.seriesIndex == 14)
                        markerString = "click for 90% boundary surface of total electron density ";   
                    else if (item.seriesIndex == 9 || item.seriesIndex == 12 || item.seriesIndex == 15)
                        markerString = "click for 95% boundary surface of total electron density ";
                    
                    
                    showTooltip(item.pageX, item.pageY, markerString);
                    //showTooltip(100, 100, markerString);
                }
            }
            else
            {
                $("#markertooltip").remove();
                previousPoint = null;            
            }
            
            if ( !isDisplayed && (pos.pageX > coords.left && pos.pageX < coords.left + plot.width()) && (pos.pageY > coords.top && pos.pageY < coords.top + plot.height())
             && (document.getElementById("monteCarloRadioButton").checked || document.getElementById("radialProbabilityDistributionCheckbox").checked) )
            {
                toggleMsgTooltip(true)
            }
            else //if ( //pos.pageX < 850 || pos.pageX > 1000)
            {
                toggleMsgTooltip(false)
            }

        }
    );
    
    function showTooltip(x, y, contents) {
        $('<div id="markertooltip">' + contents + '</div>').css( {
            position: 'absolute',
            display: 'none',
            top: y - 10,
            left: x + 5,
            border: '1px solid #fdd',
            padding: '2px',
            'background-color': FLOT_TOOLTIP_BG,
            opacity: 0.80
        }).appendTo("body").fadeIn(200);
    }
    function toggleMsgTooltip(display)
    {
        if (!isDisplayedFlotMsg && display)
        {
            $('<div id="msgtooltip">' + "click within the radial density distribution plot" + '</div>').css( {
                position: 'absolute',
                display: 'none',
                top: coords.top,
                left: coords.left,
                padding: '1px',
                'background-color': FLOT_TOOLTIP_BG,
                'font-size': '90%',
                opacity: 0.80
            }).appendTo("body").fadeIn(200);
            
            $('<div id="msgtooltip2">' + "for spherical boundary with the selected r value" + '</div>').css( {
                position: 'absolute',
                display: 'none',
                top: coords.top + 20,
                left: coords.left,
                padding: '1px',
                'background-color': FLOT_TOOLTIP_BG,
                'font-size': '90%',
                opacity: 0.80
            }).appendTo("body").fadeIn(200);
            isDisplayedFlotMsg = true;
        }
        else if (!display && isDisplayedFlotMsg)
        {
            $("#msgtooltip").remove();
            $("#msgtooltip2").remove();
            isDisplayedFlotMsg = false;
        }
    }
}


// Processing functions
function setupFlotTour()
{
    n = jmolGetN();
    l = jmolGetL();

    document.getElementById("flotExtentText").value = getExtent(n, l);
    calcJmolData(jmolGetIsosurfaceInfo().vertices, document.getElementById("manualStepSize").checked ? parseFloat(document.getElementById("stepSize").value) : null);
}



function flotTourCallback(radius)
{
    $.plot($(flotPlaceHolder),
    [
        {
            data: getPartitionedJmolData(radius),
            bars:
            {
                show: true,
                barWidth : getMCStep(),
                fill: true
            }
        }
    ],
    {
        xaxis:
        {
            min: 0,
            max: parseInt(document.getElementById("flotExtentText").value)
        }
    }
    );
}



function getExtent(n, l)
// PRE:  n and l must be valid quantum numbers where n is less than MAX_N_MAGNITUDE  NOT getting local extent var!!
// POST: FCTVAL = extent
{
    return ((n > 0 && n <= MAX_N_MAGNITUDE && l >= 0 && l < n ) ? ORBITAL_EXTENTS[n-1][l] : 0);
}


function changeExtent(changeDirection)
// PRE:  changeDirection must = DECREASE_EXTENT (-1) or INCREASE_EXTENT(1)
{
    var ex = parseInt(document.getElementById("flotExtentText").value);
    
    if (ex == 5 && changeDirection == DECREASE_EXTENT)
    {
        ex = 3;
    }
    else if (ex == 3 && changeDirection == INCREASE_EXTENT)
    {
        ex = 5;
    }
    else if(ex + changeDirection * DELTA_EXTENT > 0)
    {
        ex += changeDirection * DELTA_EXTENT;
    }
    
    document.getElementById("flotExtentText").value = parseInt(ex);
}


function addPoints(numPoints)
{   
    if(!numPoints)
        numPoints = DEFAULT_ADD_NUM_POINTS;

    this.numPoints += parseInt(numPoints);
}


function updateSeed()
{   
    seed = Math.floor(Math.random() * 100);
}




// Node menu functions

function onMouseMove(e)
{
   // It seems that any calls to change CSS must be done through jQuery in this function
   var nodeMenu = document.getElementById("nodeMenu");

   // Preserve IE compatibility
   if(!e) 
      e = window.event;

   if(isDragging)
   {
      // Move the menu by changing its top left CSS properties
      $('#nodeMenu').css('top', menuTop + e.clientY - clickY);
      $('#nodeMenu').css('left', menuLeft + e.clientX - clickX);

      // Make the menu less opaque while moving
      if(supportsRGBA())
         $('#nodeMenu').css('background', NODEMENU_BACKGROUND_MOVING);
      return true;
   }
}


function onMouseDown(e)
{
   var topelement = ("HTML" || "BODY");

   // Preserve IE compatibility
   if(!e) 
      e = window.event;

   var clickSrc = (e.target || e.srcElement);

   // Search the class name for the slider handle class. If it exists (!= -1)
   // the user is sliding the slider so the menu should be not moved.
   // Same thing with the orbital text field. It can not be clicked into and given focus
   // unless it is exempted here
   if(clickSrc.className.indexOf("ui-slider-handle") != -1 || clickSrc.className.indexOf("nodeSelect") != -1)
      return true;

   while(clickSrc.tagName != topelement && clickSrc.id !=  "nodeMenu")
      clickSrc = (clickSrc.parentNode || clickSrc.parentElement);

   if(clickSrc.id == "nodeMenu")
   {
      isDragging = true;

      menuTop = parseInt(clickSrc.offsetTop);
      menuLeft = parseInt(clickSrc.offsetLeft);

      clickX =  e.clientX;
      clickY =  e.clientY;

     document.onmousemove = onMouseMove;
     return false;
    }
}


function onMouseUp()
{
   isDragging = false;
   if(supportsRGBA())
      document.getElementById("nodeMenu").style.background = NODEMENU_BACKGROUND;
}


function displayNodeMenu()
{
   var nodeMenu = document.getElementById("nodeMenu");

   // Only display the menu if it is not already visible
   if(nodeMenu.style.display != "block") {
      setNodeMenuOpacity(0);
      nodeMenu.style.display = "block";
      for(var i=0; i<=100; i++ )
         setTimeout("setNodeMenuOpacity(" + (i/10) + ")" , 4*i);
   }
}


function setNodeMenu()
{
    var sphericalSelect = document.getElementById("sphericalTransSelect");
    var conicalSelect = document.getElementById("conicalTransSelect");
    var planarSelect = document.getElementById("planarTransSelect");

    // Remove old options
    sphericalSelect.options.length = 0;
    conicalSelect.options.length = 0;
    planarSelect.options.length = 0;

    // Add the proper number of new options, or a "-" if there are none
    if(getNumSphericalNodes() == 0)
        sphericalSelect.add(new Option("-", 0), null);
    else
        for(var i=0; i<getNumSphericalNodes(); i++)
            sphericalSelect.options[i] = new Option(i+1, i+1);

    if(getNumConicalNodes() == 0)
        conicalSelect.add(new Option("-", 0), null);
    else
        for(var i=0; i<getNumConicalNodes(); i++)
            conicalSelect.options[i] = new Option(i+1, i+1);

    if(getNumPlanarNodes() == 0)
        planarSelect.add(new Option("-", 0), null);
    else
        for(var i=0; i<getNumPlanarNodes(); i++)
            planarSelect.options[i] = new Option(i+1, i+1);


    // The sliders should be reset to their default percent
    setupSphericalNodeSlider();
    setupConicalNodeSlider();
    setupPlanarNodeSlider();
}


function closeNodeMenu()
{
   for( var i=0; i<=100; i++ ) 
      setTimeout("setNodeMenuOpacity(" + (10 - i/10) + ")" , 4*i);
   
   setTimeout("document.getElementById(\"nodeMenu\").style.display = \"none\"", 800);
}


function setNodeMenuOpacity(value)
{
   document.getElementById("nodeMenu").style.opacity = value/10;
   document.getElementById("nodeMenu").style.filter = "alpha(opacity=" + value*10 + ")";
}


function supportsRGBA()
{
    if(!("result" in arguments.callee))
    {
        var scriptElement = document.getElementsByTagName("script")[0];
        var prevColor = scriptElement.style.color;
        try
        {
            scriptElement.style.color = "rgba(0, 0, 0, 0.5)";
        } catch(e) {}
        arguments.callee.result = scriptElement.style.color != prevColor;
        scriptElement.style.color = prevColor;
    }
    return arguments.callee.result;
}


function showMLNote() {
 	alert("When you select a value for m(\u2113) that is non-zero, the probabilities will be calculated using either the sum (value > 0) or the difference (value < 0) of two inherently imaginary functions based on your selection. The sum and difference will differ only in their relation to the X and Y axes.")	  
}